/***********************************
 * Author: Matt Christian
 * 
 * Description: Creates an on-screen 3D model
 * 
 * Notes: For use with XNA Development Tutorials
 * 
 * If you use this for a project, please give credit!
 * http://matt.insidegamer.org/xnatutorials.aspx
 * 
 * ***********************************/

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace WindowsGame1
{
    class GameModel
    {
        //Class model storage
        private Model gModel;

        //Position of our model
        private Vector3 positionVector;

        /// <summary>
        /// Instantiates a new object of the GameModel class
        /// </summary>
        /// <param name="content"></param>
        /// <param name="graphics"></param>
        /// <param name="modelName"></param>
        public GameModel(ContentManager content, string modelName)
        {
            //Load model and set graphics manager
            gModel = content.Load<Model>(modelName);

            //Default position
            positionVector = new Vector3(0.0f, 0.0f, 0.0f);
        }

        /// <summary>
        /// Destructor
        /// </summary>
        ~GameModel() { }

        /// <summary>
        /// Sets the position of the model
        /// </summary>
        /// <param name="position"></param>
        public void SetPosition(Vector3 position)
        {
            //Set Position using a Vector
            positionVector = position;
        }

        /// <summary>
        /// Sets the position of the model
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="z"></param>
        public void SetPosition(float x, float y, float z)
        {
            //Set Position using floats
            positionVector.X = x;
            positionVector.Y = y;
            positionVector.Z = z;
        }

        /// <summary>
        /// Render() renders a specific model provided on class instantiation
        /// </summary>
        /// <param name="view"></param>
        /// <param name="projection"></param>
        public void Render(Matrix view, Matrix projection)
        {
            //Copy bone transformations
            Matrix[] transforms = new Matrix[gModel.Bones.Count];
            gModel.CopyAbsoluteBoneTransformsTo(transforms);

            //Render each mesh in the model
            foreach (ModelMesh mesh in gModel.Meshes)
            {
                //Determine shader information for each mesh
                foreach (BasicEffect effect in mesh.Effects)
                {
                    //Use default lighting
                    effect.EnableDefaultLighting();

                    //Set effect matrices to camera matrices
                    effect.View = view;
                    effect.Projection = projection;
                    effect.World = transforms[mesh.ParentBone.Index] * Matrix.CreateTranslation(positionVector);
                }

                //Draw mesh
                mesh.Draw();
            }
        }
    }
}
